<?xml version="1.0"?>
<xsl:stylesheet version="2.0" 
    xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
    xmlns:xs="http://www.w3.org/2001/XMLSchema">
	
<xsl:output method="html" version="4.0" encoding="UTF-8" indent="yes"/>

<xsl:param name="dateFormatter" select="'[D01].[M01].[Y]'"/>
<xsl:param name="withProjectImage"/>
<xsl:param name="withProjectStamp"/>

<xsl:template match="Project">
	<html>
	<head>
	<meta http-equiv="content-type" content="text/html; charset=utf-8" />
	<link rel="stylesheet" type="text/css" href="rsc/styles.css" />
	</head>
	<body>

	<!-- Standard Report Header for Merlin 2 Reports: START -->
	<xsl:if test="$withProjectStamp = 1">
		<div id="projectstamp">
		  <table class="report">
		  <tr>
			<th colspan="2">Projektstempel</th></tr>
			<tr class="r1">
				<td class="title">Projektname</td>
				<td class="projectname"><xsl:value-of select="title"/></td>
			</tr>
			<tr class="r0">
				<td class="title">Geplant: Start-Ende</td>
				<td>
					<xsl:value-of select="format-dateTime(plannedStartDate, $dateFormatter)"/> - 
					<xsl:value-of select="format-dateTime(plannedEndDate, $dateFormatter)"/> 
					<br />(<xsl:value-of select="days-from-duration(xs:dayTimeDuration(plannedWork/Duration/@xmlDuration))"/> Tage / 
						   <xsl:value-of select="hours-from-duration(xs:dayTimeDuration(plannedWork/Duration/@xmlDuration))"/> Std. Arbeit)
				</td>
			</tr>
			<tr class="r1">
				<td class="title">Erledigt: Start-Ende</td>
				<td>
					<xsl:value-of select="format-dateTime(expectedStartDate, $dateFormatter)"/> - 
					<xsl:value-of select="format-dateTime(expectedEndDate, $dateFormatter)"/>
					<xsl:if test="actualWork/Duration/@xmlDuration">
					<br />(<xsl:value-of select="days-from-duration(xs:dayTimeDuration(actualWork/Duration/@xmlDuration))"/> Tage /
						   <xsl:value-of select="hours-from-duration(xs:dayTimeDuration(actualWork/Duration/@xmlDuration))"/> Std. Arbeit erledigt)
					</xsl:if>
				</td>
			</tr>
			<xsl:if test="description != ''">
				<tr class="r0">
					<td class="title">Ziel</td>
					<td><xsl:value-of select="description"/></td>
				</tr>
			</xsl:if>	
		</table>
		</div>
	</xsl:if> 
	
	<xsl:if test="$withProjectImage = 1">
		<div id="logo">
			<img src="rsc/logo.png" />
		</div>
	</xsl:if> 
	<!-- Standard Report Header for Merlin 2 Reports: END -->
	
	<div id="break" />
	
	<xsl:variable name="now" select="current-date()"/>

	<h1>Wiedervorlage bis zum <xsl:value-of select="format-date($now, $dateFormatter)"/></h1>

	<div id="reviewdates">
	<table class="report">
	<tr>
	<th width="10%"><u>WV</u></th>
	<th width="15%">Typ</th>
	<th width="65%">Titel</th>
	<th width="10%">Status</th>
	</tr>

	<xsl:for-each select="//elements/*[reviewDate &lt;= xs:dateTime($now)]">
		<xsl:sort select="reviewDate"/>
		<tr class="r{position() mod 2}">
			<td><xsl:value-of select="format-dateTime(reviewDate, $dateFormatter )"/></td>
			<td>
				<xsl:value-of select="node-name(.)"/> (#<xsl:value-of select="@id"/>)
			</td>
			<td><xsl:value-of select="title"/></td>
			<td>
				<xsl:if test="flagStatus = 'red'">
					<img src="rsc/red.png" />
				</xsl:if>
				<xsl:if test="flagStatus = 'yellow'">
					<img src="rsc/yellow.png" />
				</xsl:if>
				<xsl:if test="flagStatus = 'green'">
					<img src="rsc/green.png" />
				</xsl:if>
			</td>
		</tr>
		<tr class="r{position() mod 2}">
			<td colspan="2"> </td>
		<td class="description"><xsl:value-of select="description"/></td>
		<td> </td></tr>
	</xsl:for-each>

	</table>
	</div>
	
	<div id="copyright">
		Generated by Merlin — <a href="http://www.merlin2.net/" target="_blank">www.merlin2.net</a>
	</div>

  </body>
  </html>
</xsl:template>
</xsl:stylesheet>